#!/usr/bin/env python

from ConfigParser import ConfigParser
import json
import socket
import subprocess
import sys

from charmsupport.hookenv import (
    config,
    related_units,
    relation_ids,
)


def read_config(files):
    new_config = ConfigParser()
    new_config.read(files)
    return new_config


def write_new_config(mongo_url, es_url, error_handler, charm_import_limit,
                     worker_interval, default, override, target, es_replicas,
                     es_shards):
    new_config = read_config([default, override])
    new_config.set('app:main', 'mongo.url', mongo_url)
    new_config.set('app:main', 'es_urls', es_url)
    charm_import_limit = (
        str(charm_import_limit) if charm_import_limit >= 0 else "")
    new_config.set('app:main', 'charm_import_limit', charm_import_limit)
    handler_class, handler_args = error_handler
    new_config.set('handler_exc_smtp', 'class', handler_class)
    new_config.set('handler_exc_smtp', 'args', handler_args)
    # script_lease_time specifies the time for which the scripts
    # "queue" and "review" hold their locks. Set to twice the time
    # interval execute-ingest-every, plus 1 minute slack to address
    # a possible clock skew between two instances.
    ingest_cycle = config()['execute-ingest-every']
    if ingest_cycle < 5:
        ingest_cycle = 5
    if ingest_cycle > 60:
        ingest_cycle = 60
    new_config.set(
        'app:main', 'script_lease_time', ingest_cycle)
    new_config.set(
        'app:main', 'worker_interval', worker_interval)
    new_config.set('app:main', 'es_replicas', es_replicas)
    new_config.set('app:main', 'es_shards', es_shards)
    with open(target, 'w') as output:
        new_config.write(output)


def get_relation_id_data(relation_id, unit):
    """Return a dictionary containing relation settings.

    :param relation_id: The relation id specifying the relation.
    :param unit: The name of the unit participating in the relation.
    """
    cmd = ['relation-get', '--format=json', '-r', relation_id, '-', unit]
    return json.loads(subprocess.check_output(cmd))


def get_mongo_url():
    """Use the 'database' relation settings to build a mongodb url.

    If no mongodb units are present, return an empty string.
    """
    host_ports = []
    replset = None
    for relation_id in relation_ids('database'):
        for unit in related_units(relation_id):
            relation_data = get_relation_id_data(relation_id, unit)
            # Consider configuration incomplete until port is supplied.
            if relation_data is None or 'port' not in relation_data:
                continue
            if replset is None:
                replset = relation_data['replset']
            # All units should be members of the same replication set.
            elif replset != relation_data['replset']:
                raise AssertionError('DB instances are from different sets!')
            host_ports.append(
                '%(private-address)s:%(port)s' % relation_data)
    if len(host_ports) == 0:
        return ''
    return 'mongodb://%s/?replicaSet=%s' % (','.join(host_ports), replset)


def get_elasticsearch_url():
    """Return the elasticsearch REST URL for the 'essearch' relations.

    Returns an empty string if no essearch units are present.
    """
    hostnames = []
    for relation_id in relation_ids('essearch'):
        for unit in related_units(relation_id):
            relation_data = get_relation_id_data(relation_id, unit)
            if relation_data is not None and 'host' in relation_data:
                hostnames.append('http://%s:9200/' % relation_data['host'])
    return ' '.join(hostnames)


def get_error_handler():
    """Use the 'error-email' setting to choose the error handler tuple.

    Return a tuple of logging handler class name and class's args as a string
    that can be assigned to the 'class' and 'args' keys in the INI file.
    """
    hostname = socket.gethostname()
    cmd = ['config-get', 'error-email']
    error_email = subprocess.check_output(cmd).strip()
    if error_email == '':
        # unconfigure SMTP errors.
        handler_class = 'NullHandler'
        handler_args = "()"
    else:
        handler_class = "handlers.SMTPHandler"
        values = dict(email=error_email, hostname=hostname)
        handler_args = (
            "(('localhost', 25), 'noreply@jujucharms.com', "
            "['%(email)s'], '%(hostname)s exception')" % values)
    return handler_class, handler_args


class UsageError(Exception):
    """An error triggered by incorrect usage"""


def main():
    if len(sys.argv) < 4:
        raise UsageError('Usage: %s DEFAULTS OVERRIDES TARGET')
    juju_config = config()
    charm_import_limit = juju_config['charm_import_limit']
    worker_interval = juju_config['worker-interval']
    es_shards = juju_config['es_shards']
    es_replicas = juju_config['es_replicas']

    write_new_config(get_mongo_url(), get_elasticsearch_url(),
                     get_error_handler(), charm_import_limit,
                     worker_interval, default=sys.argv[1],
                     override=sys.argv[2], target=sys.argv[3],
                     es_replicas=es_replicas, es_shards=es_shards)


if __name__ == '__main__':
    try:
        main()
    except UsageError as e:
        sys.stderr.write(str(e))
        sys.exit(1)
